#  RekallLoader.py
#  ---------------
#
#  This module provides a module loader hook so that python import's
#  can access the database scripts, obviating the need for document
#  level script lists.
#
#  Note: This code does not play well with multiple server connections
#  if the named module exists in each. We should include the server
#  connection name in the module dictionary key .....
#
import	imp
import	ihooks
import	RekallMain

#  Dictionary of loaded modules, keyed by the name. Each entry is a
#  two-element list containing the module source timestamp and the
#  module itself.
#
moduleMap = {}

class RekallLoader (ihooks.ModuleLoader) :

    #  find_module
    #  -----------
    #  Find if the module exists. Returns the module name if so.
    #
    def find_module (self, name, path = None) :

        if not RekallMain.documentExists ("script", name, "py") :
            return ihooks.ModuleLoader.find_module (self, name, path)

        return name

    #  load_module
    #  -----------
    #  Load the named module
    #
    def load_module (self, name, stuff) :

        if not RekallMain.documentExists ("script", name, "py") :
            return ihooks.ModuleLoader.load_module (self, name, stuff)

        # Get the module timestamp. If the module has been loaded
        # and the timestamp is unchanged then return said module.
        #
        stamp = RekallMain.documentStamp ("script", name, "py")
        if moduleMap.has_key(name) and moduleMap[name][0] == stamp :
            return moduleMap[name][1]

        # Load the module and store in the map before returning.
        #
        module = imp.new_module (name)
        exec RekallMain.documentContents ("script", name, "py") in module.__dict__
        moduleMap[name] = [ stamp, module ]
        return module



ihooks.install (ihooks.ModuleImporter(RekallLoader()))

