#
#   copyright            : (C)	2004,2005
#                               Mike Richardson
#   license              : This file is released under the terms of
#                          the GNU General Public License, version 2. The
#                          copyright holders retain the right to release
#                          this code under diffenent non-exclusive licences.
#   email                : mike@quaking.demon.co.uk                                     
#
#
#   Code in the module is taken from Ka-Ping Yee's cgitb.py module,
#   reformatted somewhat, and modified for Rekall. Many thanks to
#   Ka-Ping Yee for this code.
#
import	os
import	types
import	time
import	traceback
import	linecache
import	inspect
import	pydoc
import	sys


__UNDEF__ = []                          # a special sentinel object


#  strong
#  ------
#  Format text within the "strong" tag
#
def strong (text) :

    if text :
        return '<strong>' + text + '</strong>'
    return ''


#  small
#  -----
#  Format text within the "small" tag
#
def small (text) :

    if text :
        return '<small>' + text + '</small>'
    return ''


#  grey
#  ----
#  Format text as gray.
#
def grey (text) :

    if text :
        return '<font color="#909090">' + text + '</font>'
    return ''


#  lookup
#  ------
#  Look up a variable in the specified frame and locals.
#
def lookup (name, frame, locals) :

    if name in locals :
        return 'local', locals[name]

    if name in frame.f_globals :
        return 'global', frame.f_globals[name]

    if '__builtins__' in frame.f_globals :

        builtins = frame.f_globals['__builtins__']
        if type(builtins) is type({}) :
            if name in builtins :
                return 'builtin', builtins[name]
        else :
            if hasattr(builtins, name) :
                return 'builtin', getattr(builtins, name)

    return None, __UNDEF__


def scanvars (reader, frame, locals) :

    import tokenize, keyword
    vars, lasttoken, parent, prefix, value = [], None, None, '', __UNDEF__

    for ttype, token, start, end, line in tokenize.generate_tokens(reader) :

        if ttype == tokenize.NEWLINE :
            break

        if ttype == tokenize.NAME and token not in keyword.kwlist :
            if lasttoken == '.' :
                if parent is not __UNDEF__ :
                    value = getattr (parent, token, __UNDEF__)
                    vars.append ((prefix + token, prefix, value))
            else:
                where, value = lookup (token, frame, locals)
                vars.append ((token, where, value))
        elif token == '.' :
            prefix += lasttoken + '.'
            parent  = value
        else :
            parent, prefix = None, ''

        lasttoken = token

    return vars


#  html
#  ----
#  Format traceback in HTML
#
def html (etype, evalue, etb, context) :

    if type(etype) is types.ClassType :
        etype = etype.__name__

    head    = '<body bgcolor="#f0f0f8">'
    indent  = '<tt>' + small('&nbsp;' * 5) + '&nbsp;</tt>'
    frames  = []

    records = inspect.getinnerframes (etb, context)
    for frame, file, lnum, func, lines, index in records :

        file   = file or '?'
        link   = pydoc.html.escape(file)
        call   = ''

        args, varargs, varkw, locals = inspect.getargvalues(frame)

        if func != '?' :
            call = 'in ' + strong(func) + \
			inspect.formatargvalues \
			(	args,
				varargs,
				varkw,
				locals,
				formatvalue = lambda value: '=' + pydoc.html.repr(value)
			)

        hilite = {}

        def reader (lnum = [lnum]) :

            hilite[lnum[0]] = 1
            try     : return linecache.getline (file, lnum[0])
            finally : lnum[0] += 1

        vars   = scanvars(reader, frame, locals)

        rows   = [ '<tr><td bgcolor="#d8bbff"><big>&nbsp;</big>%s %s</td></tr>' % (link, call) ]

        if index is not None :

            i = lnum - index
            for line in lines :
                num  = small('&nbsp;' * (5-len(str(i))) + str(i)) + '&nbsp;'
                line = '<tt>%s%s</tt>' % (num, pydoc.html.preformat(line))
                if i in hilite :
                       rows.append ('<tr><td bgcolor="#ffccee">%s</td></tr>' % line)
                else : rows.append ('<tr><td>%s</td></tr>'                   % grey(line))
                i += 1

        done, dump = {}, []

        for name, where, value in vars :

            if name in done :
                continue

            done[name] = 1
            if value is not __UNDEF__ :
                if   where == 'global'  : name = '<em>global</em> '  + strong(name)
                elif where == 'builtin' : name = '<em>builtin</em> ' + strong(name)
                elif where == 'local'	: name = strong(name)
                else			: name = where + strong(name.split('.')[-1])
                dump.append('%s&nbsp;= %s' % (name, pydoc.html.repr(value)))
                continue

            dump.append(name + ' <em>undefined</em>')

        rows  .append ('<tr><td>%s</td></tr>' % small(grey(', '.join(dump))))
        frames.append \
		(	'''
			<table width="100%%" cellspacing=0 cellpadding=0 border=0>
			  %s
			</table>
			''' % '\n'.join(rows)
		)

        exception = \
		[	'<br/>%s: %s' % (strong(pydoc.html.escape(str(etype))),
                         pydoc.html.escape(str(evalue)))
		]

    if type(evalue) is types.InstanceType :
        for name in dir(evalue) :
            if name[:1] == '_': continue
            value = pydoc.html.repr(getattr(evalue, name))
            exception.append('\n<br/>%s%s&nbsp;=\n%s' % (indent, name, value))

    return head + ''.join(frames) + ''.join(exception) + '</body>'


#  traceback
#  ---------
#  Generate traceback in requested format.
#
def traceBack (etype, evalue, etb, context = 5) :

    try    : doc = html(etype, evalue, etb, context)
    except : doc = ''.join(traceback.format_exception(etype, evalue, etb))
    return doc


