#  ext_KBForm.py
#  -------------
#
#  The code here supports the "form.__objects__.name" mechanism for
#  getting direct access to objects by name without needing the full
#  path to the object. It replaces the KBForm.__getattr__ method to
#  trap access to the "__objects__" attribute.

#  KBFormObjectProxy
#  -----------------
#  Proxy used to field direct nide access. The __getattr__ method
#  calls the form "getGlobalCtrl()" method to locate the specified
#  node.
#
class KBFormObjectProxy :

    def __init__ (self, form) :

        self.m_form = form

    #  __getattr__
    #  -----------
    #  Locate the named object. Note that if the object is not found
    #  then thrown an exception, so it behaves like the "node.element"
    #  notation.
    #
    def __getattr__ (self, attr) :

        node = self.m_form.getGlobalCtrl (attr)
        if node is not None :
            return node

        raise AttributeError, attr


#  kbFormGetattr
#  -------------
#  Local replacement for the __getattr__ method. Intercepts if the 
#  attribute is "__objects__" and returns an instance of the class
#  "KBFormObjectProxy" which will handle object lookup.
#
def kbFormGetattr (self, attr) :

    if attr == '__objects__' :
        return KBFormObjectProxy (self)

    if self.__oldattr__ is not None :
        return self.__oldattr__ (attr)

    raise AttributeError, attr


#   Standard import code. Done this way to work both with Rekall itself
#   and with rekallWeb. Note that the original class __getattr__ method
#   is saved.
#   -------------------
#
def setAsMethods (klass) :
    try    : klass.__oldattr__ = klass.__getattr__
    except : klass.__oldattr__ = None
    setattr (klass, '__getattr__', kbFormGetattr)

try    :
    import Rekall
    setAsMethods (Rekall.PyKBForm)
except :
    pass
